"use strict";

/*
	TDPJSONCurator
	A small javascript library with a getter and a setter which curates/maintains a JSON object to a defined structure/format (i.e. it will not allow setting of propertyNames whose type or value specification does not match the defined spec)

	NOTE: currently, this is limited to single-dimension objects - i.e. it is not recursive
*/

var TDPJSONCurator=function TDPJSONCuratorFn(objectToCurateOpt, JSONObjectSpecOpt, failOnError, callback)
{

// Private vars and methods
	var curatedObject={}; // the curated/managed object
	var JSONObjectSpec=null; // the spec against which to curate/manage the object

	// Simple private function to check objectToCurate and JSONObjectSpec are objects (but not more than that is checked)
	var checkCoreObjects=function checkCoreObjectsFn(objectToCurate, JSONObjectSpec, callback)
	{
		if(typeof(objectToCurate)!=="object" || objectToCurate===null )
		{
			throw new Error("objectToCurate is not an object");
		}
		else if(typeof(JSONObjectSpec)!=="object" || JSONObjectSpec===null )
		{
			throw new Error("JSONObjectSpec is not an object");
		}
		else if(typeof(callback)!=="function")
		{
			throw new Error("callback for checkCoreObjects() is not a function");	
		}
		else
		{
			callback(true);
			return;
		}
	};

	var checkCallbackIsValid=function checkCallbackIsValidFn(callbackArgument, callback)
	{
		if(typeof(callbackArgument)!=="function")
		{		
			throw new Error("callbackArgument must be a function if supplied");
		}
		else
		{
			if(typeof(callback)==="function")
			{
				callback(true);
				return;
			}
			else
			{
				throw new Error("callback must be a function if supplied");
			}
		}
	};


// Public vars and methods
	this.set=function setFn(propertyName, value, callback)
	{
		// Private vars and methods
		var propertyName=propertyName;
		var value=value;
		var ret=false;
		var typeOK=true;
		var valueOK=true;
		var p=typeof(propertyName);
		var t=typeof(value);

		// check the callback is a function if it is supplied
		checkCallbackIsValid(callback, function()
		{
			checkCoreObjects(curatedObject, JSONObjectSpec, function()
			{
				// If a single property is being set (note that we can't check for )
				if(p==="string" && value!==undefined)
				{
					if(JSONObjectSpec[propertyName])
					{
						// Check type of param sent - if relevant JSONObjectSpec exists
						if(JSONObjectSpec[propertyName]["allowedTypes"])
						{
							if(JSONObjectSpec[propertyName]["allowedTypes"].indexOf(t)===-1)
							{
								typeOK=false;
							}
						}

						// Check value in allowed set - if relevant JSONObjectSpec exists
						if(JSONObjectSpec[propertyName]["allowedValues"])
						{
							if(JSONObjectSpec[propertyName]["allowedValues"].indexOf(value)===-1)
							{
								valueOK=false;
							}
						}
					}
					else
					{
						throw new Error(propertyName+" is not a property of JSONObjectSpec");
					}
					
					// If the type and value are OK, set the value of the propertyName as specified
					if(typeOK && valueOK)
					{
						curatedObject[propertyName]=value;

						ret=true;
					}
				}
				else
				{				
					throw new Error("propertyName must be a string");
				}


				// Return the curated object if the operation was successful
				ret=ret===true?curatedObject:false;

				// Use a provided callback if one is correctly set, otherwise simply return
				if(typeof(callback)==="function")
				{
					callback(ret);
					return;
				}
			}.bind(this));
		}.bind(this));
	};

	// Public getter - will return a single propertyName value from the JSON object if one is specified (and is valid), otherwise will return the whole object
	this.get=function getFn(propertyName, callback)
	{
		// check the callback is a function if it is supplied
		checkCallbackIsValid(callback, function()
		{
			var ret=false;

			if(typeof(propertyName)==="string" && propertyName.length)
			{
				if(propertyName in curatedObject)
				{
					ret=curatedObject[propertyName];
				}
				else
				{
					throw new Error("Property '"+propertyName+"' does not exist in curated object");
				}
			}
			else if(propertyName===undefined || propertyName===null)
			{
				ret=curatedObject;
			}

			// Use a provided callback if one is correctly set, otherwise simply return
			if(typeof(callback)==="function")
			{
				callback(ret);
				return;
			}
		}.bind(this));
	};

	
	this.curate=function curateFn(objectToCurateOpt, JSONObjectSpecOpt, failOnError, callback)
	{
		// Private methods and properties
		var objectToCurate={}; // the working object

		failOnError=!!failOnError; // Force boolean on failOnError if set, or default to true

		// Override or set objectToCurate and/or JSONObjectSpec
		if(typeof(objectToCurateOpt)==="object" && objectToCurateOpt!==null)
		{
			objectToCurate=objectToCurateOpt;

			if(typeof(JSONObjectSpecOpt)==="object" && JSONObjectSpecOpt!==null)
			{
				JSONObjectSpec=JSONObjectSpecOpt;
			}
			else
			{
				throw new Error("JSONObjectSpecOpt (object) must be provided as the second argument");
			}
		}
		else
		{
			throw new Error("objectToCurateOpt (object) must be provided as the first argument");
		}


		// check the callback is a function if it is supplied
		checkCallbackIsValid(callback, function()
		{
			checkCoreObjects(objectToCurate, JSONObjectSpec, function()
			{
				// Private vars
				var i;
				var ret=true;
				var curateFailed=false;
				var numSetsComplete=0;
				var numSetsToDo=Object.keys(objectToCurate).length;

				for(i in objectToCurate)
				{					
					this.set(i, objectToCurate[i], function(setRet)
					{
						// If the set failed and failOnError is set, mark this curation as failed and set ret to false
						if(setRet===false && failOnError)
						{
							throw new Error("Error setting property "+i+" to "+objectToCurate[i]);
						}

						numSetsComplete++;

						// If we're done setting/curating all the properties on objectToCurate
						if(numSetsComplete===numSetsToDo || curateFailed)
						{ 
							// If ret is true, return the object, otherwise return false
							if(curateFailed===true)
							{
								ret=false;
							}
							else if(ret!==false)
							{
								ret=curatedObject;
							}
							// Use a provided callback if one is correctly set, otherwise simply return
							if(typeof(callback)==="function")
							{
								callback(ret);
								return;
							}
						}
					});			
				}

			}.bind(this));
		}.bind(this));
	};


	if(typeof(callback)==="function" || callback===undefined)
	{			
		// If the constructor has had both the spec and object to curate passed in, run this.curate()
		if(objectToCurateOpt && JSONObjectSpecOpt)
		{
			this.curate(objectToCurateOpt, JSONObjectSpecOpt, failOnError, function(ret)
			{
				if(typeof(callback)==="function")
				{
					callback(ret);

					// return `this` (the instantiation of this class) to enable async constructor usage and subsequent function calling
					return this;
				}
				else
				{
					return ret;
				}
			}.bind(this));
		}
		else
		{
			return false;
		}
	}
	else
	{	
		throw new Error("Callback must be a function if supplied");
	}

};

module.exports=TDPJSONCurator