
var should=require("should");
var assert=require("assert");
var path=require("path");

var TDPJSONCurator=require(path.join(__dirname, "../TDPJSONCurator.js"));




// ASYNC TESTS
describe("Async tests", function()
{


// CONSTRUCTOR TESTS

	// INVALID objectToCurate
	describe("Invalid constructor - incorrect objectToCurate (string)", function()
	{
		it("Should throw an error when objectToCurate is not a valid object", function()
		{
			(function()
			{
				var objectToCurate=
				{
					num:100,
					str:"Some string",
					obj:
					{
						a:"b"
					},
					bool:true,
					mixed:"blah blah"
				};

				var JSONObjectSpec=
				{
					num:
					{
						allowedTypes:["number"],
						allowedValues:[1,100,1000,10000]
					},
					str:
					{
						allowedTypes:["string"]
					},
					obj:
					{
						allowedTypes:["object"]
					},
					bool:
					{
						allowedTypes:["boolean"]
					},
					mixed:
					{
						
					}
				};

				new TDPJSONCurator("abcdefg", JSONObjectSpec, true, function(ret)
				{
					
				});

			}).should.throw();
		});
	});
	
	// INVALID JSONObjectSpec
	describe("Invalid constructor - incorrect JSONObjectSpec (string)", function()
	{
		it("Should throw an error when JSONObjectSpec is not a valid object", function()
		{
			(function()
			{
				var objectToCurate=
				{
					num:100,
					str:"Some string",
					obj:
					{
						a:"b"
					},
					bool:true,
					mixed:"blah blah"
				};

				var JSONObjectSpec=
				{
					num:
					{
						allowedTypes:["number"],
						allowedValues:[1,100,1000,10000]
					},
					str:
					{
						allowedTypes:["string"]
					},
					obj:
					{
						allowedTypes:["object"]
					},
					bool:
					{
						allowedTypes:["boolean"]
					},
					mixed:
					{
						
					}
				};

				new TDPJSONCurator(objectToCurate, "abcdefgh", true, function(ret)
				{

				});

			}).should.throw();
		});
	});

	// INVALID callback
	describe("Invalid constructor - incorrect callback (string)", function()
	{
		it("Should throw an error when the callback is not a valid function", function()
		{
			(function()
			{
				var objectToCurate=
				{
					num:100,
					str:"Some string",
					obj:
					{
						a:"b"
					},
					bool:true,
					mixed:"blah blah"
				};

				var JSONObjectSpec=
				{
					num:
					{
						allowedTypes:["number"],
						allowedValues:[1,100,1000,10000]
					},
					str:
					{
						allowedTypes:["string"]
					},
					obj:
					{
						allowedTypes:["object"]
					},
					bool:
					{
						allowedTypes:["boolean"]
					},
					mixed:
					{
						
					}
				};

				new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, "stringNotAFunction");

			}).should.throw();
		});
	});

	// INVALID objectToCurate and JSONObjectSpec
	describe("Invalid constructor - incorrect JSONObjectSpec (string)", function()
	{
		it("Should throw an error when objectToCurate and JSONObjectSpec are not a valid objects", function()
		{
			(function()
			{
				var objectToCurate=
				{
					num:100,
					str:"Some string",
					obj:
					{
						a:"b"
					},
					bool:true,
					mixed:"blah blah"
				};

				var JSONObjectSpec=
				{
					num:
					{
						allowedTypes:["number"],
						allowedValues:[1,100,1000,10000]
					},
					str:
					{
						allowedTypes:["string"]
					},
					obj:
					{
						allowedTypes:["object"]
					},
					bool:
					{
						allowedTypes:["boolean"]
					},
					mixed:
					{
						
					}
				};

				new TDPJSONCurator("abcdefgh", "abcdefgh", true, function(ret)
				{

				});

			}).should.throw();
		});
	});
	

	describe("Full, working example", function()
	{
		var objectToCurate=
		{
			num:100,
			str:"Some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		// Full example, should work correctly:
		new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
		{
			it("Should return an object which is identical to that which is provided as an argument with valid constructor arguments", function()
			{		
				assert.deepEqual(ret, objectToCurate);
			});
		});
	});
	

// INVAVILD PROPERTY TESTS

	// INTEGER
	describe("Invalid property type - int, failOnError true", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:"abc",
			str:"Some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		(function()
		{
			new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
			{
				it("Should throw when failOnError is true and num is a string", function()
				{		
				});
			});
		}).should.throw();
	});


	describe("Invalid property type - int, failOnError false", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:"abc",
			str:"Some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		new TDPJSONCurator(objectToCurate, JSONObjectSpec, false, function(ret)
		{
			it("Should succeed but not set the 'num' property when failOnError is false and num is a string", function()
			{		
				ret.should.not.eql(false);
				ret.should.be.an.Object;
				assert.notDeepEqual(ret, objectToCurate);
				ret.should.not.have.property("num");
				ret.str.should.eql(objectToCurate.str);
				ret.obj.should.eql(objectToCurate.obj);
				ret.bool.should.eql(objectToCurate.bool);
				ret.mixed.should.eql(objectToCurate.mixed);
				Object.keys(ret).length.should.eql(Object.keys(objectToCurate).length - 1);
			});
		});
	});

	describe("Invalid property value - int, failOnError true", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:999,
			str:"Some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		(function()
		{
			new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
			{
				it("Should throw when failOnError is true and num is a string", function()
				{		
					ret.should.eql.false;
				});
			});
		}).should.throw();
	});


	describe("Invalid property value - int, failOnError false", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:999,
			str:"Some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		new TDPJSONCurator(objectToCurate, JSONObjectSpec, false, function(ret)
		{
			it("Should succeed but not set the 'num' property when failOnError is false and num is a string", function()
			{		
				ret.should.not.eql(false);
				ret.should.be.an.Object;;
				assert.notDeepEqual(ret, objectToCurate);
				ret.should.not.have.property("num");
				ret.str.should.eql(objectToCurate.str);
				ret.obj.should.eql(objectToCurate.obj);
				ret.bool.should.eql(objectToCurate.bool);
				ret.mixed.should.eql(objectToCurate.mixed);
				Object.keys(ret).length.should.eql(Object.keys(objectToCurate).length - 1);
			});
		});
	});


	// STRING
	describe("Invalid property type - string, failOnError true", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:100,
			str:888,
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		(function()
		{
			new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
			{
				it("Should throw when failOnError is true and str is a number", function()
				{		
					ret.should.eql.false;
				});
			});
		}).should.throw();
	});


	describe("Invalid property type - string, failOnError false", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:100,
			str:888,
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		new TDPJSONCurator(objectToCurate, JSONObjectSpec, false, function(ret)
		{
			it("Should succeed but not set the 'str' property when failOnError is false and str is a number", function()
			{		
				ret.should.not.eql(false);
				ret.should.be.an.Object;;
				assert.notDeepEqual(ret, objectToCurate);
				ret.should.not.have.property("str");
				ret.num.should.eql(objectToCurate.num);
				ret.obj.should.eql(objectToCurate.obj);
				ret.bool.should.eql(objectToCurate.bool);
				ret.mixed.should.eql(objectToCurate.mixed);
				Object.keys(ret).length.should.eql(Object.keys(objectToCurate).length - 1);
			});
		}); 
	});

	// OBJECT
	describe("Invalid property type - object, failOnError true", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:100,
			str:"some string",
			obj:777,
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		(function()
		{
			new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
			{
				it("Should throw when failOnError is true and obj is a number", function()
				{		
				});
			});
		}).should.throw();
	});


	describe("Invalid property type - object, failOnError false", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:100,
			str:"some string",
			obj:777,
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		new TDPJSONCurator(objectToCurate, JSONObjectSpec, false, function(ret)
		{
			it("Should succeed but not set the 'str' property when failOnError is false and obj is a number", function()
			{		
				ret.should.not.eql(false);
				ret.should.be.an.Object;;
				assert.notDeepEqual(ret, objectToCurate);
				ret.should.not.have.property("obj");
				ret.num.should.eql(objectToCurate.num);
				ret.str.should.eql(objectToCurate.str);
				ret.bool.should.eql(objectToCurate.bool);
				ret.mixed.should.eql(objectToCurate.mixed);
				Object.keys(ret).length.should.eql(Object.keys(objectToCurate).length - 1);
			});
		}); 
	});

	// BOOLEAN
	describe("Invalid property type - boolean, failOnError true", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:100,
			str:"some string",
			obj:
			{
				a:"b"
			},
			bool:"hello",
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		(function()
		{
			new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
			{
				it("Should throw when failOnError is true and obj is a string", function()
				{		
				});
			});
		}).should.throw();
	});


	describe("Invalid property type - boolean, failOnError false", function()
	{
		// Invalid num, should fail
		var objectToCurate=
		{
			num:100,
			str:"some string",
			obj:
			{
				a:"b"
			},
			bool:"hello",
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		new TDPJSONCurator(objectToCurate, JSONObjectSpec, false, function(ret)
		{
			it("Should succeed but not set the 'str' property when failOnError is false and obj is a string", function()
			{		
				ret.should.not.eql(false);
				ret.should.be.an.Object;;
				assert.notDeepEqual(ret, objectToCurate);
				ret.should.not.have.property("bool");
				ret.num.should.eql(objectToCurate.num);
				ret.str.should.eql(objectToCurate.str);
				ret.obj.should.eql(objectToCurate.obj);
				ret.mixed.should.eql(objectToCurate.mixed);
				Object.keys(ret).length.should.eql(Object.keys(objectToCurate).length - 1);
			});
		}); 
	});


// MORE ADVANCED VALUE CHECKING

	// STRING
	describe("Valid property allowed value - should pass", function()
	{
		// Valid value test 1, should pass
		var objectToCurate=
		{
			num:100,
			str:"some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"],
				allowedValues:["some string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
		{
			it("Should pass when string allowed value is correct", function()
			{		
				ret.should.eql.true;
			});
		});
	});

	describe("Valid property allowed value (multiple) - should pass", function()
	{
		// Valid value test 2, should pass
		var objectToCurate=
		{
			num:100,
			str:"some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"],
				allowedValues:["some string", "some other string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
		{
			it("Should pass when string allowed value is correct", function()
			{		
				ret.should.eql.true;
			});
		});
	});

	describe("Invalid property allowed value (multiple) - should fail", function()
	{
		// Invalid value test 1, should fail
		var objectToCurate=
		{
			num:100,
			str:"some string which i have appended this to",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"],
				allowedValues:["some string", "some other string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		(function()
		{
			new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
			{
				it("Should fail when string allowed value is incorrect in content", function()
				{		
				});
			});
		}).should.throw();
	});

	describe("Invalid property allowed value case (multiple) - should fail", function()
	{
		// Invalid value test 2, should fail
		var objectToCurate=
		{
			num:100,
			str:"Some string",
			obj:
			{
				a:"b"
			},
			bool:true,
			mixed:"blah blah"
		};

		var JSONObjectSpec=
		{
			num:
			{
				allowedTypes:["number"],
				allowedValues:[1,100,1000,10000]
			},
			str:
			{
				allowedTypes:["string"],
				allowedValues:["some string", "some other string"]
			},
			obj:
			{
				allowedTypes:["object"]
			},
			bool:
			{
				allowedTypes:["boolean"]
			},
			mixed:
			{
				
			}
		};

		(function()
		{
			new TDPJSONCurator(objectToCurate, JSONObjectSpec, true, function(ret)
			{
				it("Should throw when string allowed value is incorrect in case", function()
				{		
				});
			});
		}).should.throw();
	});







});


